"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SolanaWormholeCore = void 0;
const connect_sdk_1 = require("@wormhole-foundation/connect-sdk");
const connect_sdk_solana_1 = require("@wormhole-foundation/connect-sdk-solana");
const utils_1 = require("./utils");
const SOLANA_SEQ_LOG = 'Program log: Sequence: ';
class SolanaWormholeCore {
    network;
    chain;
    connection;
    contracts;
    chainId;
    coreBridge;
    constructor(network, chain, connection, contracts) {
        this.network = network;
        this.chain = chain;
        this.connection = connection;
        this.contracts = contracts;
        this.chainId = (0, connect_sdk_1.toChainId)(chain);
        const coreBridgeAddress = contracts.coreBridge;
        if (!coreBridgeAddress)
            throw new Error(`CoreBridge contract Address for chain ${chain} not found`);
        this.coreBridge = (0, utils_1.createReadOnlyWormholeProgramInterface)(coreBridgeAddress, connection);
    }
    static async fromRpc(connection, config) {
        const [network, chain] = await connect_sdk_solana_1.SolanaPlatform.chainFromRpc(connection);
        return new SolanaWormholeCore(network, chain, connection, config[chain].contracts);
    }
    publishMessage(sender, message) {
        throw new Error('Method not implemented.');
    }
    async parseTransaction(txid) {
        const response = await this.connection.getTransaction(txid);
        if (!response || !response.meta?.innerInstructions[0].instructions)
            throw new Error('transaction not found');
        const instructions = response.meta?.innerInstructions[0].instructions;
        const accounts = response.transaction.message.accountKeys;
        // find the instruction where the programId equals the Wormhole ProgramId and the emitter equals the Token Bridge
        const bridgeInstructions = instructions.filter((i) => {
            const programId = accounts[i.programIdIndex].toString();
            const wormholeCore = this.coreBridge.programId.toString();
            return programId === wormholeCore;
        });
        if (bridgeInstructions.length === 0)
            throw new Error('no bridge messages found');
        // TODO: unsure about the single bridge instruction and the [2] index, will this always be the case?
        const [logmsg] = bridgeInstructions;
        const emitterAcct = accounts[logmsg.accounts[2]];
        const emitter = (0, connect_sdk_1.toNative)(this.chain, emitterAcct.toString());
        const sequence = response.meta?.logMessages
            ?.filter((msg) => msg.startsWith(SOLANA_SEQ_LOG))?.[0]
            ?.replace(SOLANA_SEQ_LOG, '');
        if (!sequence) {
            throw new Error('sequence not found');
        }
        return [
            {
                chain: this.chain,
                emitter: emitter.toUniversalAddress(),
                sequence: BigInt(sequence),
            },
        ];
    }
}
exports.SolanaWormholeCore = SolanaWormholeCore;
//# sourceMappingURL=core.js.map